import re
import unicodedata

class IgboText():
    """The IgboText Class preprocesses texts written in Igbo language for further nlp tasks
    """
    def normalize(self, text, convert_to_lower=True, remove_abbreviations=True):
        """Normalizes a text written in Igbo Language.
        # Example Usage
            igbotext = IgboText()
            igbotext.normalize("N’ụlọ Akwụkwọ") -> n ụlọ akwụkwọ
            igbotext.normalize("N’ụlọ Akwụkwọ", False) -> N ụlọ Akwụkwọ

        # Arguments
            text: String to normalize
            convert_to_lower: Boolean toggle to either keep the upper case 
            letters or convert them to lower case
            remove_abbreviations: Toggle to remove abbreviations in the text.

        # Returns
            text: Normalized string

        # Raises
            TypeError if text is None, empty or not a string
        """
        if text is None:
            raise TypeError("The parameter text is of the type NoneType")

        if not isinstance(text, str):
            raise TypeError("The parameter text has to be a string")

        if text == "":
            raise TypeError("The parameter text is empty")

        if convert_to_lower:
            text = text.lower()
        text = self.__remove_tonal_marks__(text)
        text = self.__remove_digits_and_special_characters__(text,
                                                         remove_abbreviations)
        text = self.__split_combined_words__(text, False)
        return text.strip()

    def tokenize(self, text, convert_to_lower=False):
        """Tokenizes a text written in Igbo Language into a list of Igbo words.
        # Example Usage
            igbotext = IgboText()
            igbotext.tokenize("N’ụlọ Akwụkwọ") -> ['N’', 'ụlọ', 'Akwụkwọ']
            igbotext.normalize("N’ụlọ Akwụkwọ", True) 
            -> ['n’', 'ụlọ', 'akwụkwọ']

        # Arguments
            text: String to tokenize
            convert_to_lower: Boolean toggle to either keep the upper case 
            letters or convert them to lower case

        # Returns
            token_list: List containing tokens

        # Raises
            TypeError if text is None, empty or not a string
        """
        if text is None:
            raise TypeError("The parameter text is of the type NoneType")
            
        if not isinstance(text, str):
            raise TypeError("The parameter text has to be a string")

        if text == "":
            raise TypeError("The parameter text is empty")

        if convert_to_lower:
            text = text.lower()
        text = self.__remove_tonal_marks__(text)
        text = self.__space_out_symbols__(text)
        text = self.__split_combined_words__(text)
        token_list = text.strip().split()
        return token_list


    def __remove_tonal_marks__(self, text):
        """Removes tonal marks in the provided text.
        # Arguments
            text: String to remove tonal marks
        
        # Returns
            text: String with tonal marks removed.
        """
        text_characters = []
        for c in text:
            #Remove markings
            if unicodedata.category(c) == "Mn":
                continue
            elif ord(c) > 128 and ord(c) < 300:
                c = unicodedata.normalize('NFD', c)
                c = c.encode('ascii', 'ignore').decode('utf-8')
                text_characters.append(c)
            else:
                text_characters.append(c)
        return "".join(text_characters)

    def __remove_digits_and_special_characters__(self, text, remove_abbreviations=True):
        """Removes digits, symbols and non igbo words in the provided text.
        # Arguments
            text: String containing digits and non igbo words.
            remove_abbreviations: Toggle to remove abbreviations in the text.
        
        # Returns
            text: String with digits, symbols and non igbo words
        """
        #remove abbreviations if remove_abbreviations is true
        if remove_abbreviations:
            text = re.sub(r"(?:[a-zA-Z]\.){2,}", "", text)
        text = re.sub(r"([?.!,¿])", r" \1 ", text)
        regexp = re.compile(r"[a-zA-ZỊịṄṅỌọỤụ\-\'\’]+")
        splited_text = text.split()
        good_texts = []
        for word in splited_text:
            match = regexp.search(word)
            if match is not None and match.span()[1] == len(word):
                good_texts.append(word)
        return " ".join(good_texts)

    def __split_combined_words__(self, text, keep_symbols=True):
        """Split words combined with the hyphen and aprostrophe into two words
        # Arguments:
            text: String containing combined words.
            keep_symbols: Boolean toggle to still keep symbol or not.

        # Returns:
            text: String containing un-combined words.
        """
        if keep_symbols:
            text = re.sub(r"([-’'])", r"\1 ", text)
        else:
            text = re.sub(r"([-’'])", r" ", text)
        return text

    def __space_out_symbols__(self, text):
        """Adds space before and after some punctuation marks 
        # Arguments:
            text: String containing punctuation marks.

        # Returns:
            text: Processed string.
        """
        text = re.sub(r"([?.!:;,¿<>(){}[\]])", r" \1 ", text)
        #Remove quote found at the beginning of a word
        text = re.sub(r"\s+([-'\"‘’“”])", r" \1 ", text)
        text = re.sub(r"([-'\"‘’“”])\s+", r" \1 ", text)
        text = re.sub(r"([-'\"‘’“”])$", r" \1", text)
        text = re.sub(r"^([-'\"‘’“”])", r"\1 ", text)
        return text


import unittest
from .module import IgboText
class TestIgboText(unittest.TestCase):

    def test_remove_tonal_mark(self):
        igbo_text = IgboText()
        self.assertEqual(igbo_text.__remove_tonal_marks__("Ọ nà-ezò nnukwu mmīri n'iro"), "Ọ na-ezo nnukwu mmiri n'iro")

    def test_remove_digits_and_special_characters(self):
        igbo_text = IgboText()

        self.assertEqual(igbo_text.__remove_digits_and_special_characters__("Nnukwu mmiri na-ezo n'iro C.E.O."), "Nnukwu mmiri na-ezo n'iro")

        #Test remove_abbreviations = False
        self.assertEqual(igbo_text.__remove_digits_and_special_characters__("Nnukwu mmiri na-ezo n'iro C.E.O.", False), "Nnukwu mmiri na-ezo n'iro C E O")

    def test_split_combined_words(self):
        igbo_text = IgboText()
        self.assertEqual(igbo_text.__split_combined_words__("Ndị Fàda kwènyèrè n'atọ̀ n'ime otù."), "Ndị Fàda kwènyèrè n' atọ̀ n' ime otù.")

        #Test keep_symbol = False
        self.assertEqual(igbo_text.__split_combined_words__("Ndị Fàda kwènyèrè n'atọ̀ n'ime otù.", False), "Ndị Fàda kwènyèrè n atọ̀ n ime otù.")

    def test_space_out_symbols(self):
        igbo_text = IgboText()
        self.assertEqual(igbo_text.__space_out_symbols__("'Ọ nà-ezò nnukwu mmīri n'iro?'"), "' Ọ nà-ezò nnukwu mmīri n'iro ?  ' ")

    def test_normalize(self):
        igbo_text = IgboText()

        with self.assertRaises(TypeError):
            igbo_text.normalize("")
        
        with self.assertRaises(TypeError):
            igbo_text.normalize(None)

        self.assertEqual(igbo_text.normalize("'Ọ nà-ezò nnukwu mmīri n'iro?'"), "ọ na ezo nnukwu mmiri n iro")

        #Test convert_to_lower = False
        self.assertEqual(igbo_text.normalize("'Ọ nà-ezò nnukwu mmīri n'iro?'", False), "Ọ na ezo nnukwu mmiri n iro")

        #Test remove abbreviation = True
        self.assertEqual(igbo_text.normalize("'Ọ nà-ezò nnukwu mmīri n'iro U.S.A.?'"), "ọ na ezo nnukwu mmiri n iro")

        #Test remove abbreviation = False
        self.assertEqual(igbo_text.normalize("'Ọ nà-ezò nnukwu mmīri n'iro U.S.A.?'", remove_abbreviations=False), "ọ na ezo nnukwu mmiri n iro u s a")

        #Test convert_to_lower = False, remove abbreviation = False
        self.assertEqual(igbo_text.normalize("'Ọ nà-ezò nnukwu mmīri n'iro U.S.A.?'", convert_to_lower=False, remove_abbreviations=False), "Ọ na ezo nnukwu mmiri n iro U S A")

    def test_tokenize(self):
        igbo_text = IgboText()

        with self.assertRaises(TypeError):
            igbo_text.normalize("")
        
        with self.assertRaises(TypeError):
            igbo_text.normalize(None)

        self.assertListEqual(igbo_text.tokenize("Ndị Fàda kwènyèrè n'atọ̀ n'ime otù."), ["Ndị", "Fada", "kwenyere", "n'", "atọ", "n'", "ime", "otu", "."])

        #Test convert_to_lower = False
        self.assertListEqual(igbo_text.tokenize("Ndị Fàda kwènyèrè n'atọ̀ n'ime otù.", convert_to_lower=True), ["ndị", "fada", "kwenyere", "n'", "atọ", "n'", "ime", "otu", "."])

if __name__ == "__main__":
    unittest.main()


import setuptools

with open("README.md", "r", encoding='utf-8') as readme:
    long_description = readme.read()

setuptools.setup(
    name="igbo-text",
    version="0.1.3",
    author="Goodness Duru",
    author_email="goodyduru@gmail.com",
    description="A python package for tokenizing and normalizing texts mainly written in the Igbo Language",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/goodyduru/igbo-text",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)